@@ -16,24 +16,29 @@ class Backend:
     async def init(self):
         ...
 
+    @property
     def is_init(self):
         ...
 
-    async def close(self):
+    def close(self):
         ...
 
     async def set(
-        self, key: str, value: Any, expire: Union[None, float, int] = None, exist: Optional[bool] = None
+        self,
+        key: str,
+        value: Any,
+        expire: Union[None, float, int] = None,
+        exist: Optional[bool] = None,
     ) -> bool:
         ...
 
-    async def set_row(self, key: str, value: Any, **kwargs):
+    async def set_raw(self, key: str, value: Any, **kwargs):
         ...
 
     async def get(self, key: str, default: Optional[Any] = None) -> Any:
         ...
 
-    async def get_row(self, key: str) -> Any:
+    async def get_raw(self, key: str) -> Any:
         ...
 
     async def get_many(self, *keys: str) -> Tuple[Any]:
@@ -60,13 +65,19 @@ class Backend:
     async def get_expire(self, key: str) -> int:
         ...
 
+    async def get_bits(self, key: str, *indexes: int, size: int = 1) -> Tuple[int]:
+        ...
+
+    async def incr_bits(self, key: str, *indexes: int, size: int = 1, by: int = 1) -> Tuple[int]:
+        ...
+
     async def get_size(self, key: str) -> int:
         """
         Return size in bites that allocated by a value for given key
         """
         ...
 
-    async def ping(self, message: Optional[bytes] = None) -> str:
+    async def ping(self, message: Optional[bytes] = None) -> bytes:
         ...
 
     async def clear(self):
@@ -75,7 +86,12 @@ class Backend:
     async def set_lock(self, key: str, value: Any, expire: Union[float, int]) -> bool:
         ...
 
-    async def is_locked(self, key: str, wait: Union[None, int, float] = None, step: Union[int, float] = 0.1) -> bool:
+    async def is_locked(
+        self,
+        key: str,
+        wait: Union[None, int, float] = None,
+        step: Union[int, float] = 0.1,
+    ) -> bool:
         ...
 
     async def unlock(self, key, value) -> bool:
@@ -86,6 +102,9 @@ class Backend:
         identifier = str(uuid.uuid4())
         lock = await self.set_lock(key, identifier, expire=expire)
         if not lock:
+            # we need to check the connection by ping because
+            # if for example a redis unavailable and a backend have flag `safe`
+            # we will have a brake lock
             try:
                 await self.ping(b"TEST")
             except Exception:
@@ -109,17 +128,23 @@ class ProxyBackend(Backend):
     def is_init(self):
         return self._target.is_init
 
-    def set(self, key: str, value: Any, expire: Union[None, float, int] = None, exist: Optional[bool] = None) -> bool:
+    def set(
+        self,
+        key: str,
+        value: Any,
+        expire: Union[None, float, int] = None,
+        exist: Optional[bool] = None,
+    ) -> bool:
         return self._target.set(key, value, expire=expire, exist=exist)
 
-    def set_row(self, key: str, value: Any, **kwargs):
-        return self._target.set_row(key, value, **kwargs)
+    def set_raw(self, key: str, value: Any, **kwargs):
+        return self._target.set_raw(key, value, **kwargs)
 
     def get(self, key: str, default: Optional[Any] = None) -> Any:
         return self._target.get(key, default=default)
 
-    def get_row(self, key: str) -> Any:
-        return self._target.get_row(key)
+    def get_raw(self, key: str) -> Any:
+        return self._target.get_raw(key)
 
     def get_many(self, *keys: str) -> Tuple[Any]:
         return self._target.get_many(keys)
@@ -142,6 +167,12 @@ class ProxyBackend(Backend):
     def get_expire(self, key: str) -> int:
         return self._target.get_expire(key)
 
+    def get_bits(self, key: str, *indexes: int, size: int = 1) -> Tuple[int]:
+        return self._target.get_bits(key, *indexes, size=size)
+
+    async def incr_bits(self, key: str, *indexes: int, size: int = 1, by: int = 1) -> Tuple[int]:
+        return self._target.incr_bits(key, *indexes, size=size, by=by)
+
     def ping(self, message: Optional[bytes] = None) -> str:
         if message is not None:
             return self._target.ping(message)
@@ -156,7 +187,12 @@ class ProxyBackend(Backend):
     def set_lock(self, key: str, value: Any, expire: Union[float, int]) -> bool:
         return self._target.set_lock(key, value, expire)
 
-    def is_locked(self, key: str, wait: Union[int, float, None] = None, step: Union[int, float] = 0.1) -> bool:
+    def is_locked(
+        self,
+        key: str,
+        wait: Union[int, float, None] = None,
+        step: Union[int, float] = 0.1,
+    ) -> bool:
         return self._target.is_locked(key, wait=wait, step=step)
 
     def unlock(self, key: str, value: str) -> bool:
