@@ -8,7 +8,7 @@ from .interface import Backend
 
 __all__ = "Memory"
 
-from ..utils import _get_obj_size
+from cashews.utils import Bitarray, get_obj_size
 
 _missed = object()
 
@@ -18,16 +18,19 @@ class Memory(Backend):
     Inmemory backend lru with ttl
     """
 
+    name = "mem"
+
     def __init__(self, size: int = 1000, check_interval=1):
         self.store = OrderedDict()
         self._check_interval = check_interval
         self.size = size
         self.__is_init = False
+        self.__remove_expired_task = None
         super().__init__()
 
     async def init(self):
         self.__is_init = True
-        asyncio.create_task(self._remove_expired())
+        self.__remove_expired_task = asyncio.create_task(self._remove_expired())
 
     @property
     def is_init(self):
@@ -49,13 +52,13 @@ class Memory(Backend):
         self._set(key, value, expire)
         return True
 
-    async def set_row(self, key: str, value: Any, **kwargs):
+    async def set_raw(self, key: str, value: Any, **kwargs):
         self.store[key] = value
 
     async def get(self, key: str, default: Optional[Any] = None) -> Any:
         return self._get(key, default=default)
 
-    async def get_row(self, key: str):
+    async def get_raw(self, key: str):
         return self.store.get(key)
 
     async def get_many(self, *keys: str) -> Tuple:
@@ -106,6 +109,24 @@ class Memory(Backend):
     async def ping(self, message: Optional[bytes] = None):
         return b"PONG" if message in (None, b"PING") else message
 
+    async def get_bits(self, key: str, *indexes: int, size: int = 1) -> Tuple[int]:
+        array: Bitarray = self._get(key, default=Bitarray("0"))
+        result = []
+        for index in indexes:
+            result.append(array.get(index, size))
+        return tuple(result)
+
+    async def incr_bits(self, key: str, *indexes: int, size: int = 1, by: int = 1) -> Tuple[int]:
+        array: Bitarray = self._get(key)
+        if array is None:
+            array = Bitarray("0")
+            self._set(key, array)
+        result = []
+        for index in indexes:
+            array.incr(index, size, by)
+            result.append(array.get(index, size))
+        return tuple(result)
+
     def _set(self, key: str, value: Any, expire: Optional[float] = None):
         expire = time.time() + expire if expire else None
         if expire is None and key in self.store:
@@ -146,5 +167,10 @@ class Memory(Backend):
 
     async def get_size(self, key: str) -> int:
         if key in self.store:
-            return _get_obj_size(self.store[key])
+            return get_obj_size(self.store[key])
         return 0
+
+    def close(self):
+        if self.__remove_expired_task is not None:
+            self.__remove_expired_task.cancel()
+        super().close()
