@@ -1,9 +1,9 @@
-from .cache.defaults import CacheDetect, context_cache_detect  # noqa
+from .bloom import bloom, dual_bloom  # noqa
+from .cache.defaults import NOT_NONE, CacheDetect, context_cache_detect  # noqa
 from .cache.early import early  # noqa
 from .cache.fail import failover, fast_condition  # noqa
 from .cache.hit import hit  # noqa
 from .cache.simple import cache  # noqa
 from .circuit_breaker import CircuitBreakerOpen, circuit_breaker  # noqa
 from .locked import locked  # noqa
-from .perf import PerfDegradationException, perf  # noqa
 from .rate import RateLimitException, rate_limit  # noqa
