@@ -1,13 +1,13 @@
 import random
 from contextvars import ContextVar
-from typing import Any
+from typing import Any, Dict
 
-from ...typing import CacheCondition
+from ..._typing import CacheCondition
 
 _empty = object()
 
 
-def _default_store_condition(result, args, kwargs, key=None) -> bool:
+def _not_none_store_condition(result, args, kwargs, key=None) -> bool:
     return result is not None
 
 
@@ -15,15 +15,22 @@ def _store_all(result, args, kwargs, key=None) -> bool:
     return True
 
 
+NOT_NONE = "not_none"
+_ALL_CONDITIONS = {Any, "all", any, "any", None}
+_NOT_NONE_CONDITIONS = {NOT_NONE, "skip_none"}
+
+
 def _get_cache_condition(condition: CacheCondition):
-    if condition is None:
-        return _default_store_condition
-    if condition in [Any, "all", any, "any"]:
+    if condition in _ALL_CONDITIONS:
         return _store_all
+    if condition in _NOT_NONE_CONDITIONS:
+        return _not_none_store_condition
     return condition
 
 
 class CacheDetect:
+    __slots__ = ("_value", "_unset_token", "_previous_level")
+
     def __init__(self, previous_level=0, unset_token=None):
         self._value = {}
         self._unset_token = unset_token
@@ -33,16 +40,21 @@ class CacheDetect:
         self._value.setdefault(key, []).append(kwargs)
 
     @property
-    def keys(self):
+    def calls(self):
         return dict(self._value)
 
+    keys = calls  # backward compatibility
+
+    def clear(self):
+        self._value = {}
+
 
 _level = ContextVar("level", default=0)
 
 
 class _ContextCacheDetect:
     def __init__(self):
-        self._levels = {}
+        self._levels: Dict[int, CacheDetect] = {}
 
     @property
     def level(self):
@@ -64,7 +76,7 @@ class _ContextCacheDetect:
     def _set(self, key: str, **kwargs):
         level = self.level
         while level:
-            var: CacheDetect = self._levels.get(level)
+            var = self._levels.get(level)
             if var is None:
                 return
             var._set(key, **kwargs)
@@ -73,6 +85,7 @@ class _ContextCacheDetect:
     def _stop(self):
         if self.level in self._levels:
             token = self._levels[self.level]._unset_token
+            self._levels[self.level].clear()
             del self._levels[self.level]
             _level.reset(token)
 
