@@ -4,13 +4,15 @@ from datetime import datetime, timedelta
 from functools import wraps
 from typing import Optional
 
+from ..._typing import CacheCondition
 from ...backends.interface import Backend
-from ...key import get_cache_key, get_cache_key_template, register_template
-from ...typing import CacheCondition
+from ...formatter import register_template
+from ...key import get_cache_key, get_cache_key_template
 from .defaults import CacheDetect, _empty, _get_cache_condition, context_cache_detect
 
 __all__ = ("early",)
 logger = logging.getLogger(__name__)
+_LOCK_SUFFIX = ":lock"
 
 
 def early(
@@ -45,26 +47,53 @@ def early(
             _cache_key = get_cache_key(func, _key_template, args, kwargs)
             cached = await backend.get(_cache_key, default=_empty)
             if cached is not _empty:
-                _from_cache._set(_cache_key, ttl=ttl, name="early", backend=backend.name)
+                _from_cache._set(
+                    _cache_key,
+                    ttl=ttl,
+                    early_ttl=early_ttl,
+                    name="early",
+                    template=_key_template,
+                )
                 early_expire_at, result = cached
                 if early_expire_at <= datetime.utcnow() and await backend.set(
-                    _cache_key + ":hit", "1", expire=early_ttl, exist=False
+                    _cache_key + _LOCK_SUFFIX, "1", expire=early_ttl, exist=False
                 ):
-                    logger.info("Recalculate cache for %s (exp_at %s)", _cache_key, early_expire_at)
+                    logger.info(
+                        "Recalculate cache for %s (exp_at %s)",
+                        _cache_key,
+                        early_expire_at,
+                    )
                     asyncio.create_task(
-                        _get_result_for_early(backend, func, args, kwargs, _cache_key, ttl, early_ttl, store)
+                        _get_result_for_early(
+                            backend,
+                            func,
+                            args,
+                            kwargs,
+                            _cache_key,
+                            ttl,
+                            early_ttl,
+                            store,
+                        )
                     )
                 return result
-            return await _get_result_for_early(backend, func, args, kwargs, _cache_key, ttl, early_ttl, store)
+            return await _get_result_for_early(
+                backend, func, args, kwargs, _cache_key, ttl, early_ttl, store, unlock=True
+            )
 
         return _wrap
 
     return _decor
 
 
-async def _get_result_for_early(backend: Backend, func, args, kwargs, key, ttl: int, early_ttl: int, condition):
-    result = await func(*args, **kwargs)
-    if condition(result, args, kwargs, key):
-        early_expire_at = datetime.utcnow() + timedelta(seconds=early_ttl)
-        await backend.set(key, [early_expire_at, result], expire=ttl)
-    return result
+async def _get_result_for_early(
+    backend: Backend, func, args, kwargs, key, ttl: int, early_ttl: int, condition, unlock=False
+):
+    try:
+        result = await func(*args, **kwargs)
+        if condition(result, args, kwargs, key):
+            early_expire_at = datetime.utcnow() + timedelta(seconds=early_ttl)
+            await backend.set(key, [early_expire_at, result], expire=ttl)
+        return result
+    finally:
+        if unlock:
+            asyncio.create_task(backend.delete(key + _LOCK_SUFFIX))
