@@ -1,9 +1,10 @@
 from functools import wraps
 from typing import Optional, Tuple, Type, Union
 
+from ..._typing import CacheCondition
 from ...backends.interface import Backend
-from ...key import get_cache_key, get_cache_key_template, register_template
-from ...typing import CacheCondition
+from ...formatter import register_template
+from ...key import get_cache_key, get_cache_key_template
 from .defaults import CacheDetect, _empty, _get_cache_condition, context_cache_detect
 
 __all__ = ("failover",)
@@ -51,7 +52,13 @@ def failover(
             except exceptions as exc:
                 cached = await backend.get(_cache_key, default=_empty)
                 if cached is not _empty:
-                    _from_cache._set(_cache_key, ttl=ttl, exc=exc, name="fail", backend=backend.name)
+                    _from_cache._set(
+                        _cache_key,
+                        ttl=ttl,
+                        exc=exc,
+                        name="failover",
+                        template=_key_template,
+                    )
                     return cached
                 raise exc
             else:
