@@ -1,16 +1,21 @@
 from functools import wraps
 from typing import Optional
 
+from ..._typing import CacheCondition
 from ...backends.interface import Backend
-from ...key import get_cache_key, get_cache_key_template, register_template
-from ...typing import CacheCondition
+from ...formatter import register_template
+from ...key import get_cache_key, get_cache_key_template
 from .defaults import CacheDetect, _empty, _get_cache_condition, context_cache_detect
 
 __all__ = ("cache",)
 
 
 def cache(
-    backend: Backend, ttl: int, key: Optional[str] = None, condition: CacheCondition = None, prefix: str = "",
+    backend: Backend,
+    ttl: int,
+    key: Optional[str] = None,
+    condition: CacheCondition = None,
+    prefix: str = "",
 ):
     """
     Simple cache strategy - trying to return cached result,
@@ -32,7 +37,7 @@ def cache(
             _cache_key = get_cache_key(func, _key_template, args, kwargs)
             cached = await backend.get(_cache_key, default=_empty)
             if cached is not _empty:
-                _from_cache._set(_cache_key, ttl=ttl, name="simple", backend=backend.name)
+                _from_cache._set(_cache_key, ttl=ttl, name="simple", template=_key_template)
                 return cached
             result = await func(*args, **kwargs)
             if store(result, args, kwargs, key=_cache_key):
