@@ -1,14 +1,15 @@
 import asyncio
+from contextlib import contextmanager
 from functools import partial, wraps
 from typing import Any, Callable, Dict, Iterable, Optional, Tuple, Type, Union
 from urllib.parse import parse_qsl, urlparse
 
 from . import decorators, validation
+from ._typing import TTL, CacheCondition
 from .backends.interface import Backend
 from .backends.memory import Memory
 from .disable_control import ControlMixin, _is_disable_middleware
 from .key import ttl_to_seconds
-from .typing import TTL, CacheCondition
 
 try:
     import aioredis
@@ -21,6 +22,14 @@ else:
 
     del aioredis
 
+try:
+    import diskcache
+except ImportError:
+    DiskCache = None
+else:
+    from .backends.diskcache import DiskCache
+
+
 #  pylint: disable=too-many-public-methods
 
 
@@ -47,20 +56,41 @@ class Cache(Backend):
     default_prefix = ""
 
     def __init__(self, name=None):
-        self._backends = {}
-        self._default_middlewares = (_is_disable_middleware, _create_auto_init(), validation._invalidate_middleware)
+        self._backends = {}  # {key: (backend, middleware)}
+        self._default_middlewares = (
+            _is_disable_middleware,
+            _create_auto_init(),
+            validation._invalidate_middleware,
+        )
         self._name = name
         self._default_fail_exceptions = Exception
+        self._add_backend(Memory)
 
-    def set_default_fail_exceptions(self, exc: Union[Type[Exception], Iterable[Type[Exception]]]):
+    detect = decorators.context_cache_detect
+
+    def set_default_fail_exceptions(self, *exc: Type[Exception]):
         self._default_fail_exceptions = exc
 
     def disable(self, *cmds, prefix=""):
         return self._get_backend(prefix).disable(*cmds)
 
+    def disable_all(self, *cmds):
+        for backend, _ in self._backends.values():
+            backend.disable(*cmds)
+
     def enable(self, *cmds, prefix=""):
         return self._get_backend(prefix).enable(*cmds)
 
+    def enable_all(self, *cmds):
+        for backend, _ in self._backends.values():
+            backend.enable(*cmds)
+
+    @contextmanager
+    def disabling(self, *cmds, prefix=""):
+        self.disable(*cmds, prefix=prefix)
+        yield
+        self.enable(*cmds, prefix=prefix)
+
     def is_disable(self, *cmds, prefix=""):
         return self._get_backend(prefix).is_disable(*cmds)
 
@@ -71,9 +101,9 @@ class Cache(Backend):
         for prefix in sorted(self._backends.keys(), reverse=True):
             if key.startswith(prefix):
                 return self._backends[prefix]
-        return self._backends.get(self.default_prefix, (None, None))
+        return self._backends[self.default_prefix]
 
-    def _get_backend(self, key) -> Optional[Backend]:
+    def _get_backend(self, key) -> Backend:
         backend, _ = self._get_backend_and_config(key)
         return backend
 
@@ -94,7 +124,10 @@ class Cache(Backend):
         class _backend_class(ControlMixin, backend_class):
             pass
 
-        self._backends[prefix] = (_backend_class(**params), self._default_middlewares + middlewares)
+        self._backends[prefix] = (
+            _backend_class(**params),
+            self._default_middlewares + middlewares,
+        )
 
     async def init(self, *args, **kwargs):
         if args or kwargs:
@@ -104,29 +137,52 @@ class Cache(Backend):
 
     def _with_middlewares(self, cmd: str, key):
         backend, middlewares = self._get_backend_and_config(key)
+        return self._with_middlewares_for_backend(cmd, backend, middlewares)
+
+    def _with_middlewares_for_backend(self, cmd: str, backend, middlewares):
         call = getattr(backend, cmd)
         for middleware in middlewares:
             call = partial(middleware, call, cmd=cmd, backend=backend)
         return call
 
-    def set(self, key: str, value: Any, expire: Union[float, None, TTL] = None, exist: Optional[bool] = None):
+    def set(
+        self,
+        key: str,
+        value: Any,
+        expire: Union[float, None, TTL] = None,
+        exist: Optional[bool] = None,
+    ):
         return self._with_middlewares("set", key)(key=key, value=value, expire=ttl_to_seconds(expire), exist=exist)
 
-    def set_row(self, key: str, value: Any, **kwargs):
-        return self._with_middlewares("set_row", key)(key=key, value=value, **kwargs)
+    def set_raw(self, key: str, value: Any, **kwargs):
+        return self._with_middlewares("set_raw", key)(key=key, value=value, **kwargs)
 
     def get(self, key: str, default: Optional[Any] = None) -> Any:
         return self._with_middlewares("get", key)(key=key, default=default)
 
-    def get_row(self, key: str) -> Any:
-        return self._with_middlewares("get_row", key)(key=key)
+    def get_raw(self, key: str) -> Any:
+        return self._with_middlewares("get_raw", key)(key=key)
 
-    def keys_match(self, pattern: str):
-        return self._with_middlewares("keys_match", pattern)(pattern=pattern)
+    async def keys_match(self, pattern: str):
+        backend, middlewares = self._get_backend_and_config(pattern)
+
+        async def call(_pattern):
+            return backend.keys_match(_pattern)
+
+        for middleware in middlewares:
+            call = partial(middleware, call, cmd="keys_match", backend=backend)
+        async for key in (await call(pattern)):
+            yield key
 
     def get_many(self, *keys: str):
         return self._with_middlewares("get_many", keys[0])(*keys)
 
+    def get_bits(self, key: str, *indexes: int, size: int = 1):
+        return self._with_middlewares("get_bits", key)(key, *indexes, size=size)
+
+    def incr_bits(self, key: str, *indexes: int, size: int = 1, by: int = 1):
+        return self._with_middlewares("incr_bits", key)(key, *indexes, size=size, by=by)
+
     def incr(self, key: str) -> int:
         return self._with_middlewares("incr", key)(key=key)
 
@@ -160,32 +216,43 @@ class Cache(Backend):
 
     async def clear(self):
         for backend, _ in self._backends.values():
-            if not backend.is_init:
-                await backend.init()
-            await backend.clear()
+            await self._with_middlewares_for_backend("clear", backend, self._default_middlewares)()
 
-    async def close(self):
+    def close(self):
         for backend, _ in self._backends.values():
-            await backend.close()
+            backend.close()
 
-    def is_locked(self, key: str, wait: Union[float, None, TTL] = None, step: Union[int, float] = 0.1) -> bool:
+    def is_locked(
+        self,
+        key: str,
+        wait: Union[float, None, TTL] = None,
+        step: Union[int, float] = 0.1,
+    ) -> bool:
         return self._with_middlewares("is_locked", key)(key=key, wait=ttl_to_seconds(wait), step=step)
 
-    def _wrap_on(self, name, decorator_fabric, upper, **decor_kwargs):
-        wrapper = self._wrap_on_enable
+    def _wrap_on(self, decorator_fabric, upper, **decor_kwargs):
         if upper:
-            wrapper = self._wrap_on_enable_with_condition
-        return wrapper(name, decorator_fabric, **decor_kwargs)
+            return self._wrap_with_condition(decorator_fabric, **decor_kwargs)
+        return self._wrap(decorator_fabric, **decor_kwargs)
 
-    def _wrap_on_enable(self, name, decorator_fabric, **decor_kwargs):
+    def _wrap(self, decorator_fabric, lock=False, **decor_kwargs):
         def _decorator(func):
-            return decorator_fabric(self, **decor_kwargs)(func)
+            decorator = decorator_fabric(self, **decor_kwargs)(func)
+
+            @wraps(func)
+            async def _call(*args, **kwargs):
+                if lock:
+                    _locked = decorators.locked(self, key=decor_kwargs.get("key"), ttl=decor_kwargs["ttl"])
+                    return await _locked(decorator)(*args, **kwargs)
+                else:
+                    return await decorator(*args, **kwargs)
+
+            _call.direct = func
+            return _call
 
-        _decorator.disable = lambda: self.disable("set", prefix=name)
-        _decorator.enable = lambda: self.enable("set", prefix=name)
         return _decorator
 
-    def _wrap_on_enable_with_condition(self, name, decorator_fabric, condition, **decor_kwargs):
+    def _wrap_with_condition(self, decorator_fabric, condition, lock=False, **decor_kwargs):
         def _decorator(func):
             decorator_fabric(self, **decor_kwargs)(func)  # to register cache templates
 
@@ -199,36 +266,20 @@ class Cache(Backend):
                         return condition(result, _args, _kwargs, key=key) if condition else result is not None
 
                     decorator = decorator_fabric(self, **decor_kwargs, condition=new_condition)
-                    result = await decorator(func)(*args, **kwargs)
+                    if lock:
+                        _locked = decorators.locked(self, key=decor_kwargs.get("key"), ttl=decor_kwargs["ttl"])
+                        result = await _locked(decorator(func))(*args, **kwargs)
+                    else:
+                        result = await decorator(func)(*args, **kwargs)
 
-                    # decorators.context_cache_detect.merge(detect)
                 return result
 
+            _call.direct = func
             return _call
 
-        _decorator.disable = lambda: self.disable("set", prefix=name)
-        _decorator.enable = lambda: self.enable("set", prefix=name)
         return _decorator
 
     # DecoratorS
-    def rate_limit(
-        self,
-        limit: int,
-        period: TTL,
-        ttl: Optional[TTL] = None,
-        action: Optional[Callable] = None,
-        prefix="rate_limit",
-    ):  # pylint: disable=too-many-arguments
-        return self._wrap_on_enable(
-            "rate_limit",
-            decorators.rate_limit,
-            limit=limit,
-            period=ttl_to_seconds(period),
-            ttl=ttl_to_seconds(ttl),
-            action=action,
-            prefix=prefix,
-        )
-
     def __call__(
         self,
         ttl: TTL,
@@ -236,11 +287,12 @@ class Cache(Backend):
         condition: CacheCondition = None,
         prefix: str = "",
         upper: bool = False,
+        lock: bool = False,
     ):
         return self._wrap_on(
-            prefix or "cache",
             decorators.cache,
             upper,
+            lock=lock,
             ttl=ttl_to_seconds(ttl),
             key=key,
             condition=condition,
@@ -249,11 +301,6 @@ class Cache(Backend):
 
     cache = __call__
 
-    def invalidate(self, func, args_map: Optional[Dict[str, str]] = None, defaults: Optional[Dict] = None):
-        return self._wrap_on_enable("cache", validation.invalidate, target=func, args_map=args_map, defaults=defaults,)
-
-    invalidate_func = validation.invalidate_func
-
     def failover(
         self,
         ttl: TTL,
@@ -263,8 +310,7 @@ class Cache(Backend):
         prefix: str = "fail",
     ):
         exceptions = exceptions or self._default_fail_exceptions
-        return self._wrap_on_enable_with_condition(
-            prefix,
+        return self._wrap_with_condition(
             decorators.failover,
             ttl=ttl_to_seconds(ttl),
             exceptions=exceptions,
@@ -273,29 +319,6 @@ class Cache(Backend):
             prefix=prefix,
         )
 
-    fail = failover
-
-    def circuit_breaker(
-        self,
-        errors_rate: int,
-        period: TTL,
-        ttl: TTL,
-        exceptions: Union[Type[Exception], Tuple[Type[Exception]], None] = None,
-        key: Optional[str] = None,
-        prefix: str = "circuit_breaker",
-    ):
-        exceptions = exceptions or self._default_fail_exceptions
-        return self._wrap_on_enable(
-            prefix,
-            decorators.circuit_breaker,
-            errors_rate=errors_rate,
-            period=ttl_to_seconds(period),
-            ttl=ttl_to_seconds(ttl),
-            exceptions=exceptions,
-            key=key,
-            prefix=prefix,
-        )
-
     def early(
         self,
         ttl: TTL,
@@ -306,7 +329,6 @@ class Cache(Backend):
         upper: bool = False,
     ):
         return self._wrap_on(
-            prefix,
             decorators.early,
             upper,
             ttl=ttl_to_seconds(ttl),
@@ -320,19 +342,18 @@ class Cache(Backend):
         self,
         ttl: TTL,
         cache_hits: int,
-        update_before: int = 0,
+        update_after: int = 0,
         key: Optional[str] = None,
         condition: CacheCondition = None,
         prefix: str = "hit",
         upper: bool = False,
     ):
         return self._wrap_on(
-            prefix,
             decorators.hit,
             upper,
             ttl=ttl_to_seconds(ttl),
             cache_hits=cache_hits,
-            update_before=update_before,
+            update_after=update_after,
             key=key,
             condition=condition,
             prefix=prefix,
@@ -347,32 +368,65 @@ class Cache(Backend):
         upper: bool = False,
     ):
         return self._wrap_on(
-            prefix,
             decorators.hit,
             upper,
             ttl=ttl_to_seconds(ttl),
             cache_hits=3,
-            update_before=2,
+            update_after=1,
             key=key,
             condition=condition,
             prefix=prefix,
         )
 
-    def perf(
+    def invalidate(
         self,
-        ttl: TTL,
-        key: Optional[str] = None,
-        trace_size: int = 10,
-        perf_condition: Optional[Callable[[float, Iterable[float]], bool]] = None,
-        prefix: str = "perf",
+        func,
+        args_map: Optional[Dict[str, str]] = None,
+        defaults: Optional[Dict] = None,
     ):
-        return self._wrap_on_enable(
-            prefix,
-            decorators.perf,
+        return validation.invalidate(
+            backend=self,
+            target=func,
+            args_map=args_map,
+            defaults=defaults,
+        )
+
+    invalidate_func = validation.invalidate_func
+
+    def circuit_breaker(
+        self,
+        errors_rate: int,
+        period: TTL,
+        ttl: TTL,
+        exceptions: Union[Type[Exception], Tuple[Type[Exception]], None] = None,
+        key: Optional[str] = None,
+        prefix: str = "circuit_breaker",
+    ):
+        exceptions = exceptions or self._default_fail_exceptions
+        return decorators.circuit_breaker(
+            backend=self,
+            errors_rate=errors_rate,
+            period=ttl_to_seconds(period),
             ttl=ttl_to_seconds(ttl),
+            exceptions=exceptions,
             key=key,
-            trace_size=trace_size,
-            perf_condition=perf_condition,
+            prefix=prefix,
+        )
+
+    def rate_limit(
+        self,
+        limit: int,
+        period: TTL,
+        ttl: Optional[TTL] = None,
+        action: Optional[Callable] = None,
+        prefix="rate_limit",
+    ):  # pylint: disable=too-many-arguments
+        return decorators.rate_limit(
+            backend=self,
+            limit=limit,
+            period=ttl_to_seconds(period),
+            ttl=ttl_to_seconds(ttl),
+            action=action,
             prefix=prefix,
         )
 
@@ -383,8 +437,54 @@ class Cache(Backend):
         step: Union[int, float] = 0.1,
         prefix: str = "locked",
     ):
-        return self._wrap_on_enable(
-            prefix, decorators.locked, ttl=ttl_to_seconds(ttl), key=key, step=step, prefix=prefix
+        return decorators.locked(
+            backend=self,
+            ttl=ttl_to_seconds(ttl),
+            key=key,
+            step=step,
+            prefix=prefix,
+        )
+
+    def bloom(
+        self,
+        name: Optional[str] = None,
+        index_size: Optional[int] = None,
+        number_of_hashes: Optional[int] = None,
+        false_positives: Optional[Union[float, int]] = 1,
+        capacity: Optional[int] = None,
+        check_false_positive: bool = True,
+        prefix: str = "bloom",
+    ):
+        return decorators.bloom(
+            backend=self,
+            name=name,
+            index_size=index_size,
+            number_of_hashes=number_of_hashes,
+            false_positives=false_positives,
+            capacity=capacity,
+            check_false_positive=check_false_positive,
+            prefix=prefix,
+        )
+
+    def dual_bloom(
+        self,
+        name: Optional[str] = None,
+        index_size: Optional[int] = None,
+        number_of_hashes: Optional[int] = None,
+        false: Optional[Union[float, int]] = 1,
+        no_collisions: bool = False,
+        capacity: Optional[int] = None,
+        prefix: str = "dual_bloom",
+    ):
+        return decorators.dual_bloom(
+            backend=self,
+            name=name,
+            index_size=index_size,
+            number_of_hashes=number_of_hashes,
+            false=false,
+            no_collisions=no_collisions,
+            capacity=capacity,
+            prefix=prefix,
         )
 
 
@@ -414,14 +514,20 @@ def settings_url_parse(url):
     parse_result = urlparse(url)
     params.update(dict(parse_qsl(parse_result.query)))
     params = _fix_params_types(params)
-    if parse_result.scheme == "redis":
+    if parse_result.scheme == "redis" or parse_result.scheme == "rediss":
         if Redis is None:
             raise BackendNotAvailable("Redis backend requires `aioredis` to be installed.")
         params["backend"] = Redis
         params["address"] = parse_result._replace(query=None)._replace(fragment=None).geturl()
     elif parse_result.scheme == "mem":
         params["backend"] = Memory
+    elif parse_result.scheme == "disk":
+        if DiskCache is None:
+            raise BackendNotAvailable("Disk backend requires `diskcache` to be installed.")
+        params["backend"] = DiskCache
     elif parse_result.scheme == "":
         params["backend"] = Memory
         params["disable"] = True
+    else:
+        raise BackendNotAvailable(f"wrong backend alias {parse_result.scheme}")
     return params
