@@ -10,16 +10,26 @@ from cashews.backends.memory import Memory
 pytestmark = pytest.mark.asyncio
 
 
-@pytest.fixture(name="cache", params=["memory", pytest.param("redis", marks=pytest.mark.redis)])
-async def _cache(request, redis_dsn):
-    if request.param == "redis":
+@pytest.fixture(
+    name="cache",
+    params=[
+        "memory",
+        pytest.param("redis", marks=pytest.mark.redis),
+        pytest.param("diskcache", marks=pytest.mark.diskcache),
+    ],
+)
+async def _cache(request, redis_dsn, backend_factory):
+    if request.param == "diskcache":
+        from cashews.backends.diskcache import DiskCache
+
+        backend = await backend_factory(DiskCache, shards=0)
+        yield backend
+    elif request.param == "redis":
         from cashews.backends.redis import Redis
 
-        redis = Redis(redis_dsn, hash_key=None)
-        await redis.init()
-        await redis.clear()
-        return redis
-    return Memory()
+        yield await backend_factory(Redis, redis_dsn, hash_key=None)
+    else:
+        yield await backend_factory(Memory)
 
 
 async def test_set_get(cache):
@@ -87,9 +97,9 @@ async def test_get_set_expire(cache):
     ("method", "args", "defaults"),
     (
         ("get", ("key",), {"default": None}),
-        ("get_row", ("key",), None),
+        ("get_raw", ("key",), None),
         ("set", ("key", "value"), {"exist": None, "expire": None}),
-        ("set_row", ("key", "value"), None),
+        ("set_raw", ("key", "value"), None),
         ("incr", ("key",), None),
         ("exists", ("key",), None),
         ("delete", ("key",), None),
@@ -132,17 +142,51 @@ async def test_delete_match(cache: Backend):
     assert await cache.get("pref:test:tests") is not None
 
 
+async def test_keys_match(cache: Backend):
+    await cache.set("pref:test:test", b"value")
+    await cache.set("pref:value:test", b"value2")
+    await cache.set("pref:-:test", b"-")
+    await cache.set("pref:*:test", b"*")
+
+    await cache.set("ppref:test:test", b"value3")
+    await cache.set("pref:test:tests", b"value3")
+
+    keys = [key async for key in cache.keys_match("pref:*:test")]
+
+    assert len(keys) == 4
+
+
 async def test_get_size(cache: Backend):
     await cache.set("test", b"1")
     assert await cache.get_size("test") in (
         sys.getsizeof((None, b"1")) + sys.getsizeof(b"1") + sys.getsizeof(None),
         68,
-    )  # 106 - ordered dict,  68 redis
+        -1,
+    )  # 106 - ordered dict,  68 redis, -1 for diskcache
 
 
-async def test_lru():
-    cache = Memory(size=10)
-    # full cache
+async def test_get_bits(cache: Backend):
+    assert await cache.get_bits("test", 0, 2, 10, 50000, size=1) == (0, 0, 0, 0)
+    assert await cache.get_bits("test", 0, 1, 3, size=15) == (
+        0,
+        0,
+        0,
+    )
+
+
+async def test_incr_bits(cache: Backend):
+    await cache.incr_bits("test", 0, 1, 4)
+    assert await cache.get_bits("test", 0, 1, 2, 3, 4) == (1, 1, 0, 0, 1)
+
+
+async def test_bits_size(cache: Backend):
+    await cache.incr_bits("test", 0, 1, 4, size=5, by=3)
+    assert await cache.get_bits("test", 0, 1, 2, 3, 4, size=5) == (3, 3, 0, 0, 3)
+
+
+async def test_lru(backend_factory):
+    cache = await backend_factory(Memory, size=10)
+    # fill cache
     for i in range(10):
         await cache.set(f"key:{i}", i)
 
@@ -161,3 +205,26 @@ async def test_lru():
 
     for i in range(6, 10):
         assert await cache.get(f"key:{i}") == None
+
+
+async def test_lru2(backend_factory):
+    cache = await backend_factory(Memory, size=10)
+    # fill cache
+    for i in range(10):
+        await cache.set(f"key:{i}", i)
+
+    # use only 5 last
+    for i in range(6, 10):
+        await cache.get(f"key:{i}")
+
+    # add 5 more keys
+    for i in range(5):
+        await cache.set(f"key:{i}:new", i)
+
+    assert len(cache.store) == 10
+
+    for i in range(5):
+        assert await cache.get(f"key:{i}") == None
+
+    for i in range(6, 10):
+        assert await cache.get(f"key:{i}") == i
