@@ -3,8 +3,9 @@ from unittest.mock import Mock
 
 import pytest
 
-from cashews import decorators
+from cashews import NOT_NONE, decorators, noself
 from cashews.backends.memory import Backend, Memory
+from cashews.formatter import _REGISTER, get_templates_for_func
 
 pytestmark = pytest.mark.asyncio
 
@@ -15,18 +16,26 @@ class CustomError(Exception):
     pass
 
 
-@pytest.fixture(name="backend", params=["memory", pytest.param("redis", marks=pytest.mark.redis)])
-async def _backend(request, redis_dsn):
-    if request.param == "redis":
+@pytest.fixture(
+    name="backend",
+    params=[
+        "memory",
+        pytest.param("redis", marks=pytest.mark.redis),
+        # pytest.param("redis_cs", marks=pytest.mark.redis),
+        pytest.param("diskcache", marks=pytest.mark.diskcache),
+    ],
+)
+async def _backend(request, redis_dsn, backend_factory):
+    if request.param == "diskcache":
+        from cashews.backends.diskcache import DiskCache
+
+        yield await backend_factory(DiskCache)
+    elif request.param == "redis":
         from cashews.backends.redis import Redis
 
-        redis = Redis(redis_dsn, hash_key=None)
-        await redis.init()
-        await redis.clear()
-        return redis
-    _cache = Memory()
-    await _cache.init()
-    return _cache
+        yield await backend_factory(Redis, redis_dsn, max_connections=20)
+    else:
+        yield await backend_factory(backend_cls=Memory)
 
 
 async def test_fail_cache_simple(backend):
@@ -67,7 +76,13 @@ async def test_circuit_breaker_simple(backend):
 
 async def test_circuit_breaker_half_open(backend):
     @decorators.circuit_breaker(
-        backend, ttl=EXPIRE, half_open_ttl=0.1, errors_rate=1, min_calls=0, period=1, key="test"
+        backend,
+        ttl=EXPIRE,
+        half_open_ttl=0.1,
+        errors_rate=1,
+        min_calls=0,
+        period=1,
+        key="test",
     )
     async def func(fail=False):
         if fail:
@@ -113,10 +128,25 @@ async def test_cache_simple(backend):
     assert await func(b"notok") == b"notok"
 
 
+async def test_cache_simple_not_none(backend):
+    mock = Mock()
+
+    @decorators.cache(backend, ttl=EXPIRE, key="key", condition=NOT_NONE)
+    async def func():
+        mock()
+        return None
+
+    assert await func() is None
+    assert mock.call_count == 1
+
+    assert await func() is None
+    assert mock.call_count == 2
+
+
 async def test_cache_simple_none(backend):
     mock = Mock()
 
-    @decorators.cache(backend, ttl=EXPIRE, key="key", condition=any)
+    @decorators.cache(backend, ttl=EXPIRE, key="key")
     async def func():
         mock()
         return None
@@ -129,18 +159,50 @@ async def test_cache_simple_none(backend):
 
 
 async def test_cache_simple_key(backend):
+    _REGISTER.clear()
+
     @decorators.cache(backend, ttl=1, key="key:{some}")
-    async def func(resp=b"ok", some="err"):
+    async def func1(resp=b"ok", some="err"):
         return resp
 
-    @decorators.cache(backend, ttl=1, key="key2:{some}")
-    async def func2(resp=b"ok", some="err"):
+    @decorators.cache(backend, ttl=1)
+    async def func2(resp, some="err"):
         return resp
 
-    assert await func() == b"ok"
-    assert await func(b"notok", some="test") == b"notok"
-    assert await func(b"notok") == b"ok"
-    assert await func2(b"notok") == b"notok"
+    _noself = noself(decorators.cache)(backend, ttl=1)
+
+    class Klass:
+        @decorators.cache(backend, ttl=1)
+        async def method(self, resp):
+            return resp
+
+        @staticmethod
+        @decorators.cache(backend, ttl=1)
+        async def stat(resp):
+            return resp
+
+        @_noself
+        async def method2(self, resp):
+            return resp
+
+    await func1()
+    await func2("ok")
+    obj = Klass()
+    await obj.method("ok")
+    await obj.stat("ok")
+    await obj.method2("ok")
+
+    assert next(get_templates_for_func(func1)) == "key:{some}"
+    assert next(get_templates_for_func(func2)) == "tests.test_cache:func2:resp:{resp}:some:{some}"
+    assert (
+        next(get_templates_for_func(obj.method))
+        == "tests.test_cache:test_cache_simple_key.<locals>.Klass.method:self:{self}:resp:{resp}"
+    )
+    assert (
+        next(get_templates_for_func(obj.method2))
+        == "tests.test_cache:test_cache_simple_key.<locals>.Klass.method2:resp:{resp}"
+    )
+    assert next(get_templates_for_func(obj.stat)) == "tests.test_cache:stat:resp:{resp}"
 
 
 async def test_cache_simple_cond(backend):
@@ -177,8 +239,9 @@ async def test_early_cache_simple(backend):
     assert await func(b"notok") == b"notok"
 
 
-@pytest.mark.redis
 async def test_early_cache_parallel(backend):
+    if backend.name in ("redis", "diskcache"):
+        pytest.skip("fail in ci with slow redis or disk")
 
     mock = Mock()
 
@@ -203,15 +266,14 @@ async def test_lock_cache_parallel(backend):
     mock = Mock()
 
     @decorators.locked(backend, key="key", step=0.01)
-    async def func(resp=b"ok"):
+    async def func():
         await asyncio.sleep(0.1)
-        mock(resp)
-        return resp
+        mock()
 
-    for _ in range(4):
+    for _ in range(2):
         await asyncio.gather(*[func() for _ in range(10)], return_exceptions=True)
 
-    assert mock.call_count == 4
+    assert mock.call_count == 2
 
 
 async def test_lock_cache_parallel_ttl(backend):
@@ -266,48 +328,22 @@ async def test_hit_cache(backend):
 async def test_hit_cache_early(backend):
     mock = Mock()
 
-    @decorators.hit(backend, ttl=10, cache_hits=1, key="test", update_before=0)
+    @decorators.hit(backend, ttl=10, cache_hits=5, key="test", update_after=1)
     async def func(resp=b"ok"):
         mock(resp)
         return resp
 
-    assert await func(b"1") == b"1"  # cache
+    assert await func(b"1") == b"1"  # nocache
     assert mock.call_count == 1
 
-    assert await func(b"2") == b"1"  # cache
+    assert await func(b"2") == b"1"  # cache and update
     assert mock.call_count == 1
 
     await asyncio.sleep(0.01)
-    assert await func(b"3") == b"2"  # cache
     assert mock.call_count == 2
-
-
-async def test_perf_cache(backend):
-    mock = Mock()
-
-    @decorators.perf(backend, key="test", ttl=0.2)
-    async def func(s=0.01):
-        await asyncio.sleep(s)
-        mock()
-        return "res"
-
-    await asyncio.gather(*[func() for _ in range(10)])
-    assert mock.call_count == 10
-    await func(0.04)
-    assert mock.call_count == 11
-    with pytest.raises(decorators.PerfDegradationException):
-        await func(0.001)  # long
-        assert mock.call_count == 11
-    await asyncio.sleep(0.05)
-    # prev was slow so no hits
-    with pytest.raises(decorators.PerfDegradationException):
-        await asyncio.gather(*[func() for _ in range(100)])
-
-    assert mock.call_count == 11
-    await asyncio.sleep(0.2)
-
-    await func(0.009)
-    assert mock.call_count == 12
+    assert await func(b"3") == b"2"  # cache from prev and also update
+    await asyncio.sleep(0.01)
+    assert mock.call_count == 3
 
 
 async def test_cache_detect_simple(backend):
